(document:surround "/std/frame")
(document:insert "/std/functions")

(po-domain "alterator-standalone")

;;; Functions
(define centerface (make-widget "centerface"))
(define modules (make-attribute 'modules))
(define sections (make-attribute 'sections))
(define current-module (make-attribute 'current-module))

title (_ "System management center")
width 600
height 560

(set-global! 'expert_mode 0)
(define *helps* (make-cell '()))

(define *mode-onchange* (make-cell #f))
(set-global! 'frame:mode-onchange-set!
             (lambda (cb)
               (cell-set! *mode-onchange* cb)))
(define (frame:mode-onchange)
  (define-operation get-context)
  (let ((mode-onchange (cell-ref *mode-onchange*))
        (page (global 'frame:page)))
    (if (and (procedure? mode-onchange)
             page)
        (with-context (get-context page)
                      (mode-onchange)))))

(define (not-empty-section category module-data)
  (find (lambda(x) (string=? category (woo-get-option x 'category)))
	module-data))

(define (make-section module-data)
  (lambda(x)
    (let ((category (woo-get-option x 'category)))
      (and (not-empty-section category module-data)
	   (list category
		 (woo-get-option x 'name)
		 (woo-get-option x 'comment)
		 (woo-get-option x 'icon))))))

(define (make-module x)
  (list (woo-get-option x 'category)
	(woo-get-option x 'uri)
	(woo-get-option x 'name)))

(define (make-help data)
  (cell-set! *helps*
    (map (lambda(x)
           (cons (woo-get-option x 'uri)
	         (woo-get-option x 'help)))
	 data)))

(define (update-menu)
  (let ((section-data (woo-list "/menu/avail_sections" 'ui "qt" 'expert_mode (global 'expert_mode)))
	(module-data  (woo-list "/menu/avail_modules"  'ui "qt" 'expert_mode (global 'expert_mode))))
      (make-help module-data)
      (cf sections (filter-map (make-section module-data) section-data)
          modules  (map make-module module-data))))

(define (update-help url)
 (let* ((topic (if (eq? url '/) "acc-qt" (cond-assoc (symbol->string url) (cell-ref *helps*))))
        (path  (woo-get-option (woo-read-first "/help/" 'topic topic) 'url)))
  (document:root help path)))


;;; UI
(document:id cf (centerface (frame:page)
  (when clicked
    (let ((action (cf current-action))
          (old-mode (global 'expert_mode)))
      (case action
        ((base_mode) (set-global! 'expert_mode 0)
                     (if (not (= old-mode 0))
                       (frame:mode-onchange))
                     (update-menu))
        ((expert_mode) (set-global! 'expert_mode 1)
                       (if (= old-mode 0)
                         (frame:mode-onchange))
                       (update-menu)))))
  (when selected
    (let ((url (cf current-module)))
      (frame:replace url)
      (update-help url)))))

;;; Logic

(document:root (when loaded
		 (update-menu)
		 (update-help '/)))
